@@ -136,7 +136,8 @@ static int cmd_subvol_create(int argc, char **argv)
 	retval = 1;	/* failure */
 	res = test_isdir(dst);
 	if (res < 0 && res != -ENOENT) {
-		error("cannot access %s: %s", dst, strerror(-res));
+		errno = -res;
+		error("cannot access %s: %m", dst);
 		goto out;
 	}
 	if (res >= 0) {
@@ -155,7 +156,7 @@ static int cmd_subvol_create(int argc, char **argv)
 	}
 
 	len = strlen(newname);
-	if (len == 0 || len >= BTRFS_VOL_NAME_MAX) {
+	if (len > BTRFS_VOL_NAME_MAX) {
 		error("subvolume name too long: %s", newname);
 		goto out;
 	}
@@ -337,8 +338,8 @@ again:
 	} else if (commit_mode == COMMIT_AFTER) {
 		res = get_fsid(dname, fsid, 0);
 		if (res < 0) {
-			error("unable to get fsid for '%s': %s",
-				path, strerror(-res));
+			errno = -res;
+			error("unable to get fsid for '%s': %m", path);
 			error(
 			"delete succeeded but commit may not be done in the end");
 			ret = 1;
@@ -690,7 +691,8 @@ static int cmd_subvol_snapshot(int argc, char **argv)
 
 	res = test_isdir(dst);
 	if (res < 0 && res != -ENOENT) {
-		error("cannot access %s: %s", dst, strerror(-res));
+		errno = -res;
+		error("cannot access %s: %m", dst);
 		goto out;
 	}
 	if (res == 0) {
@@ -715,7 +717,7 @@ static int cmd_subvol_snapshot(int argc, char **argv)
 	}
 
 	len = strlen(newname);
-	if (len == 0 || len >= BTRFS_VOL_NAME_MAX) {
+	if (len > BTRFS_VOL_NAME_MAX) {
 		error("snapshot name too long '%s'", newname);
 		goto out;
 	}
@@ -910,13 +912,13 @@ static int cmd_subvol_find_new(int argc, char **argv)
 }
 
 static const char * const cmd_subvol_show_usage[] = {
-	"btrfs subvolume show [options] <subvol-path>|<mnt>",
-	"Show more information about the subvolume",
+	"btrfs subvolume show [options] <path>",
+	"Show more information about the subvolume (UUIDs, generations, times, snapshots)",
 	"-r|--rootid   rootid of the subvolume",
 	"-u|--uuid     uuid of the subvolume",
 	"",
-	"If no option is specified, <subvol-path> will be shown, otherwise",
-	"the rootid or uuid are resolved relative to the <mnt> path.",
+	"If no option is specified, subvolume at <path> will be shown, otherwise",
+	"the rootid or uuid are resolved relative to the <path>.",
 	NULL
 };
 
